@@ -5,23 +5,23 @@
 #
 
 package PDL::PP::Code;
-use Carp;
-our @CARP_NOT;
 
 use strict;
+use warnings;
+use Carp;
 
 sub get_pdls {my($this) = @_; return ($this->{ParNames},$this->{ParObjs});}
 
-my @code_args_always = qw(SignatureObj GenericTypes ExtraGenericLoops HaveThreading Name);
+my @code_args_always = qw(BadFlag SignatureObj GenericTypes ExtraGenericSwitches HaveThreading Name);
 sub make_args {
   my ($which) = @_;
-  ("Parsed$which", [$which,"_Bad$which",@code_args_always]);
+  ("Parsed$which", [$which,\"Bad$which",@code_args_always]);
 }
 
 # Do the appropriate substitutions in the code.
 sub new {
     my($class,$code,$badcode,
-       $sig,$generictypes,$extrageneric,$havethreading,$name,
+       $handlebad, $sig,$generictypes,$extrageneric,$havethreading,$name,
        $dont_add_thrloop, $backcode ) = @_;
     my $parnames = $sig->names_sorted;
 
@@ -30,8 +30,7 @@ sub new {
     confess "Error: empty or undefined GenericTypes!\n"
       unless @{$generictypes || []};
 
-    # simple way of handling bad code check
-    my $handlebad = defined($badcode);
+    $badcode //= $code if $handlebad;
 
     # last two arguments may not be supplied
     #
@@ -75,7 +74,7 @@ sub new {
     # loops (array references, 1. item = variable.
     #
     my ( $threadloops, $coderef, $sizeprivs ) =
-	$this->separate_code( "{$code\n}" );
+	$this->separate_code( "{\n$code\n}" );
 
     # Now, if there is no explicit threadlooping in the code,
     # enclose everything into it.
@@ -93,10 +92,10 @@ sub new {
     #
     # NOTE: amalgamate sizeprivs from good and bad code
     #
-    if ( $handlebad ) {
+    if ( $handlebad && ($code ne $badcode || $badcode =~ /PDL_BAD_CODE|PDL_IF_BAD/) ) {
 	print "Processing 'bad' code...\n" if $::PP_VERBOSE;
 	my ( $bad_threadloops, $bad_coderef, $bad_sizeprivs ) =
-	    $this->separate_code( "{$badcode\n}" );
+	    $this->separate_code( "{\n$badcode\n}" );
 
 	if(!$bad_threadloops && !$dont_add_thrloop) {
 	    print "Adding 'bad' threadloop...\n" if $::PP_VERBOSE;
@@ -129,7 +128,7 @@ sub new {
 
     # Enclose it all in a genericloop.
     my $nc = $coderef;
-    $coderef = PDL::PP::GenericLoop->new($generictypes, undef,
+    $coderef = PDL::PP::GenericSwitch->new($generictypes, undef,
 	  [grep {!$extrageneric->{$_}} @$parnames],'$PRIV(__datatype)');
     push @{$coderef},$nc;
 
@@ -142,7 +141,7 @@ sub new {
     my $no = 0;
     for(sort keys %glh) {
 	my $nc = $coderef;
-	$coderef = PDL::PP::GenericLoop->new($generictypes,$no++,
+	$coderef = PDL::PP::GenericSwitch->new($generictypes,$no++,
 					    $glh{$_},$_);
 	push @$coderef,$nc;
     }
@@ -153,13 +152,12 @@ sub new {
     $this->{Code} = (join '',sort values %$sizeprivs).
        ($dont_add_thrloop?'':PDL::PP::pp_line_numbers __LINE__, join "\n",
         'PDL_COMMENT("threadloop declarations")',
+        'int __thrloopval;',
         'register PDL_Indx __tind0,__tind1; PDL_COMMENT("counters along dim")',
         'register PDL_Indx __tnpdls = $PRIV(pdlthread).npdls;',
         'PDL_COMMENT("dims here are how many steps along those dims")',
-        (map "register PDL_Indx __tinc0_$_ = \$PRIV(pdlthread).incs[$_];", 0..$#$parnames),
-        (map "register PDL_Indx __tinc1_$_ = \$PRIV(pdlthread).incs[__tnpdls+$_];", 0.. $#$parnames),
-        $this->threadloop_start,
-        $this->threadloop_end,
+        (map "register PDL_Indx __tinc0_$parnames->[$_] = PDL_THR_INC(\$PRIV(pdlthread).incs,__tnpdls,$_,0);", 0..$#$parnames),
+        (map "register PDL_Indx __tinc1_$parnames->[$_] = PDL_THR_INC(\$PRIV(pdlthread).incs,__tnpdls,$_,1);", 0..$#$parnames),
        ).
        $this->params_declare.
        join('',map $_->get_incregisters, @$pobjs{sort keys %$pobjs}).
@@ -182,51 +180,30 @@ EOF
 }
 
 sub func_name { $_[1] ? "writebackdata" : "readdata" }
-sub threadloop_start_name { "PDL_STARTTHREADLOOP_$_[0]{Name}" }
-sub threadloop_end_name { "PDL_ENDTHREADLOOP_$_[0]->{Name}" }
 
 sub threadloop_start {
-    my ($this) = @_;
+    my ($this, $funcname) = @_;
     my ($ord,$pdls) = $this->get_pdls;
-    my $macro_name = $this->threadloop_start_name;
-    PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-#define $macro_name(funcName) \\
-if ( PDL->startthreadloop(&(\$PRIV(pdlthread)),\$PRIV(vtable)->funcName, __privtrans) ) return; \\
-       do { \\
-	    PDL_Indx *__tdims = PDL->get_threaddims(&\$PRIV(pdlthread)); \\
-	    register PDL_Indx __tdims1 = __tdims[1]; \\
-	    register PDL_Indx __tdims0 = __tdims[0]; \\
-	    register PDL_Indx *__offsp = PDL->get_threadoffsp(&\$PRIV(pdlthread)); \\
-      PDL_COMMENT("incs are each pdl's stride, declared at func start") \\
-      PDL_COMMENT("offs are each pthread's starting offset into each pdl") \\
-@{[ join " \\\n", map $pdls->{$ord->[$_]}->do_pointeraccess." += __offsp[$_];", 0..$#$ord ]} \\
-      for( __tind1 = 0 ; \\
-	    __tind1 < __tdims1 ; \\
-	    __tind1++ \\
-	    PDL_COMMENT("step by tinc1, undoing inner-loop of tinc0*tdims0") \\
-@{[ join " \\\n", map "\t\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc1_$_ - __tinc0_$_ * __tdims0", 0..$#$ord ]} \\
-	 ) \\
-      { \\
-	 for( __tind0 = 0 ; \\
-	      __tind0 < __tdims0 ; \\
-	      __tind0++ \\
-@{[ join " \\\n", map "\t\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc0_${_}", 0..$#{$ord} ]} \\
-	   ) { \\
-      PDL_COMMENT("This is the tightest threadloop. Make sure inside is optimal.")
+    <<EOF;
+PDL_THREADLOOP_START(
+$funcname,
+\$PRIV(pdlthread),
+\$PRIV(vtable),
+@{[ join "", map "\t".$pdls->{$ord->[$_]}->do_pointeraccess." += __offsp[$_];\n", 0..$#$ord ]},
+(@{[ join "", map "\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc1_$ord->[$_] - __tinc0_$ord->[$_] * __tdims0\n", 0..$#$ord ]}),
+(@{[ join "", map "\t,".$pdls->{$ord->[$_]}->do_pointeraccess." += __tinc0_$ord->[$_]\n", 0..$#{$ord} ]})
+)
 EOF
 }
 
 sub threadloop_end {
     my ($this) = @_;
     my ($ord,$pdls) = $this->get_pdls();
-    my $macro_name = $this->threadloop_end_name;
-    PDL::PP::pp_line_numbers(__LINE__, <<EOF);
-#define $macro_name \\
-} \\
-} \\
-PDL_COMMENT("undo outer-loop of tinc1*tdims1, and original per-pthread offset") \\
-@{[ join " \\\n", map $pdls->{$ord->[$_]}->do_pointeraccess." -= __tinc1_$_ * __tdims1 + __offsp[$_];", 0..$#$ord ]} \\
-} while(PDL->iterthreadloop(&\$PRIV(pdlthread),2));
+    <<EOF;
+PDL_THREADLOOP_END(
+\$PRIV(pdlthread),
+@{[ join "", map $pdls->{$ord->[$_]}->do_pointeraccess." -= __tinc1_$ord->[$_] * __tdims1 + __offsp[$_];\n", 0..$#$ord ]}
+)
 EOF
 }
 
@@ -244,6 +221,54 @@ sub make_loopind { my($this,$ind) = @_;
 	return [$ind,$orig];
 }
 
+my %access2class = (
+  GENERIC => 'PDL::PP::GentypeAccess',
+  PPSYM => 'PDL::PP::PpsymAccess',
+);
+
+sub process {
+    my ($this, $code, $stack_ref, $threadloops_ref, $sizeprivs) = @_;
+    while($code) {
+	# Parse next statement
+	$code =~ s/^(.*?) # First, some noise is allowed. This may be bad.
+	    ( \$(ISBAD|ISGOOD|SETBAD)\s*\(\s*\$?[a-zA-Z_]\w*\s*\([^)]*\)\s*\)   # $ISBAD($a(..)), ditto for ISGOOD and SETBAD
+	        |\$[a-zA-Z_]\w*\s*\([^)]*\)  # $a(...): access
+		|\bloop\s*\([^)]+\)\s*%\{   # loop(..) %{
+		|\btypes\s*\([^)]+\)\s*%\{  # types(..) %{
+		|\bthreadloop\s*%\{         # threadloop %{
+		|%}                        # %}
+		|$)//xs
+		    or confess("Invalid program $code");
+	my $control = $2;
+	# Store the user code.
+	# Some day we shall parse everything.
+	push @{$stack_ref->[-1]},$1;
+	# Then, our control.
+	if (!$control) { print("No \$2!\n") if $::PP_VERBOSE; next; }
+	if($control =~ /^loop\s*\(([^)]+)\)\s*%\{/) {
+	    my $ob = PDL::PP::Loop->new([split ',',$1], $sizeprivs,$this);
+	    print "SIZEPRIVSXX: $sizeprivs,",(join ',',%$sizeprivs),"\n" if $::PP_VERBOSE;
+	    push @{$stack_ref->[-1]},$ob;
+	    push @$stack_ref,$ob;
+	} elsif($control =~ /^types\s*\(([^)]+)\)\s*%\{/) {
+	    my $ob = PDL::PP::Types->new($1,$this);
+	    push @{$stack_ref->[-1]},$ob;
+	    push @$stack_ref,$ob;
+	} elsif($control =~ /^threadloop\s*%\{/) {
+	    my $ob = PDL::PP::ThreadLoop->new;
+	    push @{$stack_ref->[-1]},$ob;
+	    push @$stack_ref,$ob;
+	    $$threadloops_ref++;
+	} elsif($control =~ /^%}/) {
+	    pop @$stack_ref;
+	} else {
+	    my ($rest, @add) = $this->expand($control.$code);
+	    push @{$stack_ref->[-1]}, @add;
+	    $code = $rest;
+	}
+    } # while: $code
+}
+
 # my ( $threadloops, $coderef, $sizeprivs ) = $this->separate_code( $code );
 #
 # separates the code into an array of C fragments (strings),
@@ -254,71 +279,44 @@ sub separate_code {
     my ( $this, $code ) = @_;
     # First check for standard code errors:
     catch_code_errors($code);
-    my $coderef = PDL::PP::Block->new;
-    my @stack = ($coderef);
+    my @stack = my $coderef = PDL::PP::Block->new;
     my $threadloops = 0;
     my $sizeprivs = {};
-    local $_ = $code;
-    while($_) {
-	# Parse next statement
-	# I'm not convinced that having the checks twice is a good thing,
-	# since it makes it easy (for me at least) to forget to update one
-	# of them
-	s/^(.*?) # First, some noise is allowed. This may be bad.
-	    ( \$(ISBAD|ISGOOD|SETBAD)\s*\(\s*\$?[a-zA-Z_]\w*\s*\([^)]*\)\s*\)   # $ISBAD($a(..)), ditto for ISGOOD and SETBAD
-                |\$PP(ISBAD|ISGOOD|SETBAD)\s*\(\s*[a-zA-Z_]\w*\s*,\s*[^)]*\s*\)   # $PPISBAD(CHILD,[1]) etc
-###                |\$STATE(IS|SET)(BAD|GOOD)\s*\(\s*[^)]*\s*\)      # $STATEISBAD(a) etc
-                |\$PDLSTATE(IS|SET)(BAD|GOOD)\s*\(\s*[^)]*\s*\)   # $PDLSTATEISBAD(a) etc
-	        |\$[a-zA-Z_]\w*\s*\([^)]*\)  # $a(...): access
-		|\bloop\s*\([^)]+\)\s*%\{   # loop(..) %{
-		|\btypes\s*\([^)]+\)\s*%\{  # types(..) %{
-		|\bthreadloop\s*%\{         # threadloop %{
-		|%}                        # %}
-		|$)//xs
-		    or confess("Invalid program $_");
-	my $control = $2;
-	# Store the user code.
-	# Some day we shall parse everything.
-	push @{$stack[-1]},$1;
-	if ( $control =~ /^\$STATE/ ) { print "\nDBG: - got [$control]\n\n"; }
-	# Then, our control.
-	if($control) {
-	    if($control =~ /^loop\s*\(([^)]+)\)\s*%\{/) {
-		my $ob = PDL::PP::Loop->new([split ',',$1], $sizeprivs,$this);
-		print "SIZEPRIVSXX: $sizeprivs,",(join ',',%$sizeprivs),"\n" if $::PP_VERBOSE;
-		push @{$stack[-1]},$ob;
-		push @stack,$ob;
-	    } elsif($control =~ /^types\s*\(([^)]+)\)\s*%\{/) {
-		my $ob = PDL::PP::Types->new($1,$this);
-		push @{$stack[-1]},$ob;
-		push @stack,$ob;
-	    } elsif($control =~ /^threadloop\s*%\{/) {
-		my $ob = PDL::PP::ThreadLoop->new;
-		push @{$stack[-1]},$ob;
-		push @stack,$ob;
-		$threadloops ++;
-	    } elsif($control =~ /^\$PP(ISBAD|ISGOOD|SETBAD)\s*\(\s*([a-zA-Z_]\w*)\s*,\s*([^)]*)\s*\)/) {
-		push @{$stack[-1]},PDL::PP::PPBadAccess->new($1,$2,$3,$this);
-	    } elsif($control =~ /^\$(ISBAD|ISGOOD|SETBAD)VAR\s*\(\s*([^)]*)\s*,\s*([^)]*)\s*\)/) {
-		push @{$stack[-1]},PDL::PP::BadVarAccess->new($1,$2,$3,$this);
-	    } elsif($control =~ /^\$(ISBAD|ISGOOD|SETBAD)\s*\(\s*\$?([a-zA-Z_]\w*)\s*\(([^)]*)\)\s*\)/) {
-		push @{$stack[-1]},PDL::PP::BadAccess->new($1,$2,$3,$this);
-	    } elsif($control =~ /^\$PDLSTATE(IS|SET)(BAD|GOOD)\s*\(\s*([^)]*)\s*\)/) {
-		push @{$stack[-1]},PDL::PP::PDLStateBadAccess->new($1,$2,$3,$this);
-	    } elsif($control =~ /^\$[a-zA-Z_]\w*\s*\([^)]*\)/) {
-		push @{$stack[-1]},PDL::PP::Access->new($control,$this);
-	    } elsif($control =~ /^%}/) {
-		pop @stack;
-	    } else {
-		confess("Invalid control: $control\n");
-	    }
-	} else {
-	    print("No \$2!\n") if $::PP_VERBOSE;
-	}
-    } # while: $_
+    $this->process($code, \@stack, \$threadloops, $sizeprivs);
     ( $threadloops, $coderef, $sizeprivs );
 } # sub: separate_code()
 
+sub expand {
+    my ($this, $text) = @_;
+    my (undef, $pdl, $inds, $rest) = PDL::PP::Rule::Substitute::macro_extract($text);
+    my @add;
+    if($pdl =~ /^T/) {@add = PDL::PP::MacroAccess->new($pdl,$inds,
+			   $this->{Generictypes},$this->{Name});}
+    elsif(my $c = $access2class{$pdl}) {@add = $c->new($pdl,$inds)}
+    elsif($pdl =~ /^(PP|)(ISBAD|ISGOOD|SETBAD)(VAR|)$/) {
+	my ($opcode, $name) = ($2);
+	my $get = $1 || $3;
+	if (!$get) {
+	    $inds =~ s/^\$?([a-zA-Z_]\w*)\s*//; # $ is optional
+	    $name = $1;
+	    $inds = substr $inds, 1, -1; # chop off brackets
+	} elsif ($get eq 'PP') {
+	    ($name, $inds) = split /\s*,\s*/, $inds;
+	} else {
+	    ($inds, $name) = $inds =~ /(.*)\s*,\s*(\w+)/;
+	}
+	@add = PDL::PP::BadAccess->new($opcode,$get,$name,$inds,$this);
+    }
+    elsif($this->{ParObjs}{$pdl}) {@add = PDL::PP::Access->new($pdl,$inds)}
+    else {
+	@add = "\$$pdl(";
+	# assumption: the only "control" that will happen in macro args is another macro
+	$this->process($inds, [\@add], undef, undef);
+	push @add, ")";
+    }
+    ($rest, @add);
+}
+
 # This is essentially a collection of regexes that look for standard code
 # errors and croaks with an explanation if they are found.
 sub catch_code_errors {
@@ -352,16 +350,6 @@ sub report_error {
     die "$message at $filename line $line\n";
 }
 
-###########################
-#
-# used by BadAccess code to know when to use NaN support
-# - since 2.040, use per-PDL code
-#
-sub convert {
-    my ( $this, $name, $lhs, $rhs, $opcode, $pobj ) = @_;
-    ($lhs, $rhs);
-}
-
 #####################################################################
 #
 # Encapsulate the parsing code objects
@@ -370,24 +358,19 @@ sub convert {
 # 	new - constructor
 #	get_str - get the string to be put into the xsub.
 
-###########################
-#
-# Encapsulate a block
-
 package PDL::PP::Block;
 
 sub new { my($type) = @_; bless [],$type; }
 
 sub myoffs { 0 }
 sub myprelude {}
-sub myitem { "" }
 sub mypostlude {}
 
 sub get_str {
     my ($this,$parent,$context) = @_;
     my $str = $this->myprelude($parent,$context);
-    $str .= $this->get_str_int($parent,$context);
-    $str .= $this->mypostlude($parent,$context);
+    $str .= $this->get_str_int($parent,$context)//'';
+    $str .= $this->mypostlude($parent,$context)//'';
     return $str;
 }
 
@@ -396,10 +379,11 @@ sub get_str_int {
   my $nth=0;
   my $str = "";
   MYLOOP: while(1) {
-    my $it = $this->myitem($parent,$nth);
+    my $it = $this->can('myitemstart') && $this->myitemstart($parent,$nth);
     last MYLOOP if $nth and !$it;
-    $str .= $it;
+    $str .= $it//'';
     $str .= join '', $this->get_contained($parent,$context);
+    $str .= $it if $it = $this->can('myitemend') && $this->myitemend($parent,$nth);
     $nth++;
   }
   return $str;
@@ -432,18 +416,18 @@ sub get_str {
     my $str = PDL::PP::pp_line_numbers(__LINE__, <<EOF);
 if ( \$PRIV(bvalflag) ) { PDL_COMMENT("** do 'bad' Code **")
 #define PDL_BAD_CODE
+#define PDL_IF_BAD(t,f) t
   @{[ $bad->get_str($parent,$context) ]}
 #undef PDL_BAD_CODE
+#undef PDL_IF_BAD
 } else { PDL_COMMENT("** else do 'good' Code **")
+#define PDL_IF_BAD(t,f) f
   @{[ $good->get_str($parent,$context) ]}
+#undef PDL_IF_BAD
 }
 EOF
 }
 
-###########################
-#
-# Encapsulate a loop
-
 package PDL::PP::Loop;
 our @ISA = "PDL::PP::Block";
 
@@ -479,10 +463,7 @@ sub mypostlude { my($this,$parent,$context) = @_;
 	return join '', map PDL::PP::pp_line_numbers(__LINE__-1, "}} PDL_COMMENT(\"Close $_\")"), @{$this->[0]};
 }
 
-###########################
-#
-# Encapsulate a generic type loop
-package PDL::PP::GenericLoop;
+package PDL::PP::GenericSwitch;
 our @ISA = "PDL::PP::Block";
 
 # make the typetable from info in PDL::Types
@@ -505,17 +486,15 @@ sub myoffs {4}
 sub myprelude {
     my ($this,$parent,$context) = @_;
     push @{$parent->{Gencurtype}}, undef; # so that $GENERIC can get at it
-    die "ERROR: need to rethink NaN support in GenericLoop\n"
+    die "ERROR: need to rethink NaN support in GenericSwitch\n"
 	if defined $this->[1] and $parent->{ftypes_type};
-    <<WARNING_EATER;
-PDL_COMMENT("Start generic loop")
-	switch($this->[3]) { case -42: PDL_COMMENT("Warning eater") {(void)1;
-WARNING_EATER
+    qq[PDL_COMMENT("Start generic loop")\n\tswitch($this->[3]) {\n];
 }
 
-sub myitem {
+my @GENTYPE_ATTRS = qw(integer real unsigned);
+sub myitemstart {
     my ($this,$parent,$nth) = @_;
-    my $item = $this->[0]->[$nth] || return "";
+    my $item = $this->[0][$nth] || return "";
     $parent->{Gencurtype}[-1] = $item;
     @$parent{qw(ftypes_type ftypes_vars)} = ($item, $this->[2]) if defined $this->[1];
     my ($ord,$pdls) = $parent->get_pdls;
@@ -524,29 +503,40 @@ sub myitem {
       ? PDL::PP::pp_line_numbers(__LINE__-1, "\t\tPDL_DECLARE_PARAMS_$parent->{Name}(@{[join ',', @param_ctypes]})\n")
       : join '', map $_->get_xsdatapdecl($_->adjusted_type($item)->ctype),
           map $parent->{ParObjs}{$_}, sort keys %{$this->[2]};
+    my @gentype_decls = map "#define PDL_IF_GENTYPE_".uc($_)."(t,f) ".
+	($item->$_ ? 't' : 'f')."\n",
+	@GENTYPE_ATTRS;
     join '',
-	PDL::PP::pp_line_numbers(__LINE__-1, "\t} break; case @{[$item->sym]}: {\n"),
+	PDL::PP::pp_line_numbers(__LINE__-1, "case @{[$item->sym]}: {\n"),
+	@gentype_decls,
 	$decls;
 }
 
+sub myitemend {
+    my ($this,$parent,$nth) = @_;
+    my $item = $this->[0][$nth] || return "";
+    join '',
+	"\n",
+	(map "#undef PDL_IF_GENTYPE_".uc($_)."\n", @GENTYPE_ATTRS),
+	PDL::PP::pp_line_numbers(__LINE__-1, "} break;\n");
+}
+
 sub mypostlude {
     my($this,$parent,$context) = @_;
     pop @{$parent->{Gencurtype}};  # and clean up the Gentype stack
     $parent->{ftypes_type} = undef if defined $this->[1];
     my $supported = join '', map $_->ppsym, @{$this->[0]};
-    "\tbreak;}
-	default:barf(\"PP INTERNAL ERROR in $parent->{Name}: unhandled datatype(%d), only handles ($supported)! PLEASE MAKE A BUG REPORT\\n\", $this->[3]);}\n";
+    "\n\tdefault:return PDL->make_error(PDL_EUSERERROR, \"PP INTERNAL ERROR in $parent->{Name}: unhandled datatype(%d), only handles ($supported)! PLEASE MAKE A BUG REPORT\\n\", $this->[3]);}\n";
 }
 
 ####
 #
-# This relies on PP.pm making sure that initthreadloop always sets
+# This relies on PP.pm making sure that initthreadstruct always sets
 # up the two first dimensions even when they are not necessary.
 #
 package PDL::PP::ThreadLoop;
 use Carp;
 our @ISA = "PDL::PP::Block";
-our @CARP_NOT;
 
 sub new {
 	my $type   = shift;
@@ -555,12 +545,12 @@ sub new {
 sub myoffs { return 0; }
 sub myprelude {
     my($this,$parent,$context, $backcode) = @_;
-    $parent->threadloop_start_name.'('.$parent->func_name($backcode).')';
+    $parent->threadloop_start($parent->func_name($backcode));
 }
 
 # Should possibly fold out thread.dims[0] and [1].
 sub mypostlude {my($this,$parent,$context) = @_;
-    $parent->threadloop_end_name;
+    $parent->threadloop_end;
 }
 
 # Simple subclass of ThreadLoop to implement writeback code
@@ -569,7 +559,6 @@ sub mypostlude {my($this,$parent,$context) = @_;
 package PDL::PP::BackCodeThreadLoop;
 use Carp;
 our @ISA = "PDL::PP::ThreadLoop";
-our @CARP_NOT;
 
 sub myprelude {
     my($this,$parent,$context, $backcode) = @_;
@@ -590,7 +579,6 @@ package PDL::PP::Types;
 use Carp;
 use PDL::Types ':All';
 our @ISA = "PDL::PP::Block";
-our @CARP_NOT;
 my %types = map +($_=>1), ppdefs_all; # BSUL....
 
 sub new {
@@ -609,34 +597,11 @@ sub get_str {
 }
 
 
-###########################
-#
-# Encapsulate an access
-
 package PDL::PP::Access;
 use Carp;
-our @CARP_NOT;
 
-my %access2class = (
-  P => 'PDL::PP::PointerAccess',
-  PP => 'PDL::PP::PhysPointerAccess',
-  SIZE => 'PDL::PP::SizeAccess',
-  GENERIC => 'PDL::PP::GentypeAccess',
-  PPSYM => 'PDL::PP::PpsymAccess',
-  PDL => 'PDL::PP::PdlAccess',
-);
-
-sub new { my($type,$str,$parent) = @_;
-    $str =~ /^\$([a-zA-Z_]\w*)\s*\(([^)]*)\)/ or
-	    confess ("Access wrong: '$str'\n");
-    my($pdl,$inds) = ($1,$2);
-    if($pdl =~ /^T/) {PDL::PP::MacroAccess->new($pdl,$inds,
-			   $parent->{Generictypes},$parent->{Name});}
-    elsif(my $c = $access2class{$pdl}) {$c->new($pdl,$inds)}
-    elsif(!defined $parent->{ParObjs}{$pdl}) {PDL::PP::OtherAccess->new($pdl,$inds);}
-    else {
-	bless [$pdl,$inds],$type;
-    }
+sub new { my($type,$pdl,$inds,$parent) = @_;
+    bless [$pdl,$inds],$type;
 }
 
 sub get_str { my($this,$parent,$context) = @_;
@@ -645,329 +610,55 @@ sub get_str { my($this,$parent,$context) = @_;
 }
 
 ###########################
-#
-# Just some other substituted thing.
-
-package PDL::PP::OtherAccess;
-sub new { my($type,$pdl,$inds) = @_; bless [$pdl,$inds],$type; }
-sub get_str {my($this) = @_;return "\$$this->[0]($this->[1])"}
-
-###########################
-#
 # Encapsulate a check on whether a value is good or bad
 # handles both checking (good/bad) and setting (bad)
-#
-# Integer types (BSUL) + floating point when no NaN (FD)
-#   $ISBAD($a(n))  -> $a(n) == a_badval
-#   $ISGOOD($a())     $a()  != a_badval
-#   $SETBAD($a())     $a()   = a_badval
-#
-# floating point with NaN
-#   $ISBAD($a(n))  -> !isnan($a(n)) == 0
-#   $ISGOOD($a())     !isnan($a())  != 0
-#   $SETBAD($a())     $a()           = PDL->bvals.Float (or .Double)
-#
-# I've also got it so that the $ on the pdl name is not
-# necessary - so $ISBAD(a(n)) is also accepted, so as to reduce the
-# amount of line noise. This is actually done by the regexp
-# in the separate_code() sub at the end of the file.
-#
-# note:
-#   we also expand out $a(n) etc as well here
-#
-# To do:
-#   need to allow use of F,D without NaN
-#
-
 package PDL::PP::BadAccess;
 use Carp;
-our @CARP_NOT;
 
 sub new {
-    my ( $type, $opcode, $name, $inds, $parent ) = @_;
-
-    # trying to avoid auto creation of hash elements
-    my $check = $parent->{ParObjs};
+    my ( $type, $opcode, $get, $name, $inds, $parent ) = @_;
     die "\nIt looks like you have tried a \$${opcode}() macro on an\n" .
 	"  unknown ndarray <$name($inds)>\n"
-	unless exists($check->{$name}) and defined($check->{$name});
-
-    return bless [$opcode, $name, $inds], $type;
+	unless defined($parent->{ParObjs}{$name});
+    bless [$opcode, $get, $name, $inds], $type;
 }
 
-our %ops = ( ISBAD => '==', ISGOOD => '!=', SETBAD => '=' );
+sub _isbad { "PDL_ISBAD($_[0],$_[1],$_[2])" }
+our %ops = (
+    ISBAD => \&_isbad,
+    ISGOOD => sub {'!'.&_isbad},
+    SETBAD => sub{join '=', @_[0,1]},
+);
+my %getters = (
+    '' => sub {my ($obj, $inds, $context)=@_; $obj->do_access($inds,$context)},
+    PP => sub {my ($obj, $inds)=@_; $obj->do_physpointeraccess.$inds},
+    VAR => sub {my ($obj, $inds)=@_; $inds},
+);
 
 sub get_str {
-    my($this,$parent,$context) = @_;
-
-    my $opcode = $this->[0];
-    my $name   = $this->[1];
-    my $inds   = $this->[2];
-
+    my ($this,$parent,$context) = @_;
+    my ($opcode, $get, $name, $inds) = @$this;
+    confess "generic type access outside a generic loop in $name"
+      unless defined $parent->{Gencurtype}[-1];
     print "PDL::PP::BadAccess sent [$opcode] [$name] [$inds]\n" if $::PP_VERBOSE;
-
-    my $op = $ops{$opcode};
     die "ERROR: unknown check <$opcode> sent to PDL::PP::BadAccess\n"
-	unless defined $op;
-
-    my $obj = $parent->{ParObjs}{$name};
+	unless defined( my $op = $ops{$opcode} );
     die "ERROR: something screwy in PDL::PP::BadAccess (PP/PDLCode.pm)\n"
-	unless defined( $obj );
-
-    my ( $lhs, $rhs ) = $parent->convert(
-	$name, $obj->do_access($inds,$context), "${name}_badval", $opcode, $obj
-    );
-
+	unless defined( my $obj = $parent->{ParObjs}{$name} );
+    my $lhs = $getters{$get}->($obj, $inds, $context);
+    my $rhs = "${name}_badval";
     print "DBG:  [$lhs $op $rhs]\n" if $::PP_VERBOSE;
-    return "$lhs $op $rhs";
+    my $type = exists $parent->{ftypes_vars}{$name}
+	? $parent->{ftypes_type}
+	: $obj->adjusted_type($parent->{Gencurtype}[-1]);
+    $op->($lhs, $rhs, $type->ppsym);
 }
 
 
-###########################
-#
-# Encapsulate a check on whether a value is good or bad
-# handles both checking (good/bad) and setting (bad)
-#
-# Integer types (BSUL) + floating point when no NaN (FD)
-#   $ISBADVAR(foo,a)  -> foo == a_badval
-#   $ISGOODVAR(foo,a)    foo != a_badval
-#   $SETBADVAR(foo,a)    foo  = a_badval
-#
-# floating point with NaN
-#   $ISBADVAR(foo,a)  -> !isnan(foo) == 0
-#   $ISGOODVAR(foo,a)    !isnan(foo) != 0
-#   $SETBADVAR(foo,a)    foo          = PDL->bvals.Float (or .Double)
-#
-
-package PDL::PP::BadVarAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new {
-    my ( $type, $opcode, $var_name, $name, $parent ) = @_;
-
-    # trying to avoid auto creation of hash elements
-    my $check = $parent->{ParObjs};
-    die "\nIt looks like you have tried a \$${opcode}() macro on an\n" .
-	"  unknown ndarray <$name>\n"
-	unless exists($check->{$name}) and defined($check->{$name});
-
-    bless [$opcode, $var_name, $name], $type;
-}
-
-our %ops = ( ISBAD => '==', ISGOOD => '!=', SETBAD => '=' );
-
-sub get_str {
-    my($this,$parent,$context) = @_;
-
-    my $opcode   = $this->[0];
-    my $var_name = $this->[1];
-    my $name = $this->[2];
-
-    print "PDL::PP::BadVarAccess sent [$opcode] [$var_name] [$name]\n" if $::PP_VERBOSE;
-
-    my $op = $ops{$opcode};
-    die "ERROR: unknown check <$opcode> sent to PDL::PP::BadVarAccess\n"
-	unless defined $op;
-
-    my $obj = $parent->{ParObjs}{$name};
-    die "ERROR: something screwy in PDL::PP::BadVarAccess (PP/PDLCode.pm)\n"
-	unless defined( $obj );
-
-    my ( $lhs, $rhs ) = $parent->convert(
-	$name, $var_name, "${name}_badval", $opcode, $obj
-    );
-
-    print "DBG:  [$lhs $op $rhs]\n" if $::PP_VERBOSE;
-    return "$lhs $op $rhs";
-}
-
-
-###########################
-#
-# Encapsulate a check on whether a value is good or bad using PP
-# handles both checking (good/bad) and setting (bad)
-
-# this is only an initial attempt - it will, almost certainly,
-# need more work as more code is converted to handle bad values
-#
-# currently it can only handle cases like
-#  $PPISBAD(PARENT,[i])  -> PARENT_physdatap[i] == PARENT_badval
-#  etc
-#
-# if we use NaN's, then
-#  $PPISBAD(PARENT,[i])   -> !isnan(PARENT_physdatap[i]) == 0
-#  $PPISGOOD(PARENT,[i])  -> !isnan(PARENT_physdatap[i]) != 0
-#  $PPSETBAD(PARENT,[i])  -> PARENT_physdatap[i]          = PDL->bvals.Float (or .Double)
-#
-
-package PDL::PP::PPBadAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new {
-    my ( $type, $opcode, $name, $inds, $parent ) = @_;
-
-    $opcode =~ s/^PP//;
-    bless [$opcode, $name, $inds], $type;
-}
-
-# PP is stripped in new()
-our %ops = ( ISBAD => '==', ISGOOD => '!=', SETBAD => '=' );
-
-sub get_str {
-    my($this,$parent,$context) = @_;
-
-    my $opcode = $this->[0];
-    my $name   = $this->[1];
-    my $inds   = $this->[2];
-
-    print "PDL::PP::PPBadAccess sent [$opcode] [$name] [$inds]\n" if $::PP_VERBOSE;
-
-    my $op = $ops{$opcode};
-    die "\nERROR: unknown check <$opcode> sent to PDL::PP::PPBadAccess\n"
-	unless defined $op;
-
-    my $obj = $parent->{ParObjs}{$name};
-    die "\nERROR: ParObjs does not seem to exist for <$name> = problem in PDL::PP::PPBadAccess\n"
-	unless defined $obj;
-
-    my ( $lhs, $rhs ) = $parent->convert(
-	$name, $obj->do_physpointeraccess() . "$inds", "${name}_badval", $opcode, $obj
-    );
-
-    print "DBG:  [$lhs $op $rhs]\n" if $::PP_VERBOSE;
-    return "$lhs $op $rhs";
-}
-
-
-###########################
-#
-# Encapsulate a check on whether the state flag of an ndarray
-# is set/change this state
-#
-# $PDLSTATEISBAD(a)    ->  ($PDL(a)->state & PDL_BADVAL) > 0
-# $PDLSTATEISGOOD(a)   ->  ($PDL(a)->state & PDL_BADVAL) == 0
-#
-# $PDLSTATESETBAD(a)   ->  ($PDL(a)->state |= PDL_BADVAL)
-# $PDLSTATESETGOOD(a)  ->  ($PDL(a)->state &= ~PDL_BADVAL)
-#
-
-package PDL::PP::PDLStateBadAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new {
-    my ( $type, $op, $val, $name, $parent ) = @_;
-
-    # $op  is one of: IS SET
-    # $val is one of: GOOD BAD
-
-    # trying to avoid auto creation of hash elements
-    my $check = $parent->{ParObjs};
-    die "\nIt looks like you have tried a \$PDLSTATE${op}${val}() macro on an\n" .
-	"  unknown ndarray <$name>\n"
-	unless exists($check->{$name}) and defined($check->{$name});
-
-    bless [$op, $val, $name], $type;
-}
-
-our %ops  = (
-	     IS  => { GOOD => '== 0', BAD => '> 0' },
-	     SET => { GOOD => '&= ~', BAD => '|= ' },
-	     );
-
-sub get_str {
-    my($this,$parent,$context) = @_;
-
-    my $op   = $this->[0];
-    my $val  = $this->[1];
-    my $name = $this->[2];
-
-    print "PDL::PP::PDLStateBadAccess sent [$op] [$val] [$name]\n" if $::PP_VERBOSE;
-
-    my $opcode = $ops{$op}{$val};
-    my $type = $op . $val;
-    die "ERROR: unknown check <$type> sent to PDL::PP::PDLStateBadAccess\n"
-	unless defined $opcode;
-
-    my $obj = $parent->{ParObjs}{$name};
-    die "\nERROR: ParObjs does not seem to exist for <$name> = problem in PDL::PP::PDLStateBadAccess\n"
-	unless defined $obj;
-
-    my $state = $obj->do_pdlaccess() . "->state";
-
-    my $str;
-    if ( $op eq 'IS' ) {
-	$str = "($state & PDL_BADVAL) $opcode";
-    } elsif ( $op eq 'SET' ) {
-	$str = "$state ${opcode}PDL_BADVAL";
-    }
-
-    print "DBG:  [$str]\n" if $::PP_VERBOSE;
-    return $str;
-}
-
-
-###########################
-#
-# Encapsulate a Pointeraccess
-
-package PDL::PP::PointerAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
-
-sub get_str {my($this,$parent,$context) = @_;
-	confess ("can't access undefined pdl ".$this->[0])
-	  unless defined($parent->{ParObjs}{$this->[0]});
-#	$parent->{ParObjs}{$this->[0]}->{FlagPaccess} = 1;
-	$parent->{ParObjs}{$this->[0]}->{FlagPhys} = 1;
-	$parent->{ParObjs}{$this->[0]}->do_pointeraccess();
-}
-
-
-###########################
-#
-# Encapsulate a PhysPointeraccess
-
-package PDL::PP::PhysPointerAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
-
-sub get_str {my($this,$parent,$context) = @_;
-	$parent->{ParObjs}{$this->[0]}->do_physpointeraccess()
-	 if defined($parent->{ParObjs}{$this->[0]});
-}
-
-###########################
-#
-# Encapsulate a PDLaccess
-
-package PDL::PP::PdlAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
-
-sub get_str {my($this,$parent,$context) = @_;
-	confess ("can't access undefined pdl ".$this->[0])
-	  unless defined($parent->{ParObjs}{$this->[0]});
-	$parent->{ParObjs}{$this->[0]}->do_pdlaccess();
-}
-
-###########################
-#
-# Encapsulate a macroaccess
-
 package PDL::PP::MacroAccess;
 use Carp;
 use PDL::Types ':All';
 my $types = join '',ppdefs_all;
-our @CARP_NOT;
 
 sub new {
     my ($type, $pdl, $inds, $gentypes, $name) = @_;
@@ -977,7 +668,7 @@ sub new {
     my @lst = split ',', $inds, -1;
     confess "Macroaccess: different nos of args $pdl $inds\n" if @lst != @ilst;
     my %type2value; @type2value{@ilst} = @lst;
-    warn "$name has no Macro for generic type $_ (has $pdl)\n"
+    confess "$name has no Macro for generic type $_ (has $pdl)\n"
 	for grep !exists $type2value{$_}, @$gentypes;
     my %gts; @gts{@$gentypes} = ();
     warn "Macro for unsupported generic type identifier $_\n"
@@ -993,30 +684,8 @@ sub get_str {
     $type2value->{$parent->{Gencurtype}[-1]->ppsym};
 }
 
-
-###########################
-#
-# Encapsulate a SizeAccess
-
-package PDL::PP::SizeAccess;
-use Carp;
-our @CARP_NOT;
-
-sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
-
-sub get_str {my($this,$parent,$context) = @_;
-	confess "can't get SIZE of undefined dimension $this->[0]"
-	  unless defined($parent->{IndObjs}{$this->[0]});
-	$parent->{IndObjs}{$this->[0]}->get_size();
-}
-
-###########################
-#
-# Encapsulate a GentypeAccess
-
 package PDL::PP::GentypeAccess;
 use Carp;
-our @CARP_NOT;
 
 sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
 
@@ -1030,7 +699,6 @@ sub get_str {my($this,$parent,$context) = @_;
 
 package PDL::PP::PpsymAccess;
 use Carp;
-our @CARP_NOT;
 
 sub new { my($type,$pdl,$inds) = @_; bless [$inds],$type; }
 
