@@ -9,11 +9,11 @@ my $prefile = "";
    $prefile = <>;
 }
 
-my ($postfile) = &PDL::NiceSlice::perldlpp("PDL::NiceSlice", $prefile);
+my ($postfile) = PDL::NiceSlice->perldlpp($prefile);
 
 print $postfile;
 
-END
+__END__
 
 =head2 perldlpp.pl
 
